import os
import ftplib
import xml.etree.ElementTree as ET
import xbmc
import xbmcaddon
import xbmcgui
import xbmcvfs

# Einstellungen laden
ADDON = xbmcaddon.Addon()
ENABLED = ADDON.getSettingBool('enable_sync')
FTP_BASE_PATH = ADDON.getSettingString('ftp_base_path')
CUSTOM_FOLDER = ADDON.getSettingString('custom_folder')
SYNC_STANDARD = ADDON.getSettingBool('sync_standard')
SYNC_STATIC = ADDON.getSettingBool('sync_static')
SERVER_PRIORITY = ADDON.getSettingBool('server_priority')
FTP_HOST = ADDON.getSettingString('ftp_host')
FTP_USER = ADDON.getSettingString('ftp_user')
FTP_PASS = ADDON.getSettingString('ftp_pass')
FTP_PATH = f"/{FTP_BASE_PATH}/auto_fav_sync/{CUSTOM_FOLDER}/favourites.xml"
LOCAL_FAVOURITES = os.path.join(xbmcvfs.translatePath('special://userdata'), 'favourites.xml')
SUPER_FAVOURITES_PATH = os.path.join(xbmcvfs.translatePath('special://userdata'), 'addon_data', 'plugin.program.super.favourites', 'Super Favourites')
TEMP_DOWNLOAD_PATH = os.path.join(xbmcvfs.translatePath('special://temp'), 'favourites_download.xml')

STATIC_FOLDERS = ['Anime', 'Horror', 'Marvel', 'Goat']

# Mehrsprachigkeit
LANGUAGE = ADDON.getLocalizedString

# Pfad zum Icon
ICON_PATH = os.path.join(ADDON.getAddonInfo('path'), 'resources', 'images', 'icon.png')

def show_notification(message_id, duration=5000, **kwargs):
    message = LANGUAGE(message_id).format(**kwargs)
    xbmc.executebuiltin(f'Notification(Auto FTP Sync, {message}, {duration}, {ICON_PATH})')

def ftp_upload(local_path, remote_path):
    try:
        with ftplib.FTP(FTP_HOST) as ftp:
            ftp.login(FTP_USER, FTP_PASS)
            with open(local_path, 'rb') as file:
                ftp.storbinary(f'STOR {remote_path}', file)
        return True
    except Exception as e:
        xbmc.log(f"FTP upload failed: {str(e)}", xbmc.LOGERROR)
        return False

def ftp_download(remote_path, local_path):
    try:
        with ftplib.FTP(FTP_HOST) as ftp:
            ftp.login(FTP_USER, FTP_PASS)
            with open(local_path, 'wb') as file:
                ftp.retrbinary(f'RETR {remote_path}', file.write)
        return True
    except Exception as e:
        xbmc.log(f"FTP download failed: {str(e)}", xbmc.LOGERROR)
        return False

def ftp_folder_exists(folder_path):
    try:
        with ftplib.FTP(FTP_HOST) as ftp:
            ftp.login(FTP_USER, FTP_PASS)
            ftp.cwd(folder_path)
        return True
    except ftplib.error_perm as e:
        if '550' in str(e):
            return False
        else:
            xbmc.log(f"FTP error: {str(e)}", xbmc.LOGERROR)
            return False

def create_ftp_folder(folder_path):
    try:
        with ftplib.FTP(FTP_HOST) as ftp:
            ftp.login(FTP_USER, FTP_PASS)
            ftp.mkd(folder_path)
        return True
    except Exception as e:
        xbmc.log(f"FTP create folder failed: {str(e)}", xbmc.LOGERROR)
        return False

def merge_favourites(local_path, download_path, folder_name):
    try:
        if not os.path.exists(local_path):
            show_notification(30005, 5000)  # Ordner enthält keine Favoriten
            return False

        local_tree = ET.parse(local_path)
        local_root = local_tree.getroot()
        
        if not os.path.exists(download_path):
            show_notification(30006, 5000)  # Keine Remote-Favoriten gefunden
            return False

        download_tree = ET.parse(download_path)
        download_root = download_tree.getroot()

        local_fav_set = {fav.get('name') for fav in local_root.findall('favourite')}
        download_fav_set = {fav.get('name') for fav in download_root.findall('favourite')}

        # Hinzufügen von Einträgen, die in der lokalen Datei vorhanden sind, aber nicht in der heruntergeladenen Datei
        for fav in local_root.findall('favourite'):
            name = fav.get('name')
            if name not in download_fav_set:
                download_root.append(fav)

        # Entfernen von Einträgen, die in der heruntergeladenen Datei vorhanden sind, aber nicht in der lokalen Datei
        for fav in download_root.findall('favourite'):
            name = fav.get('name')
            if name not in local_fav_set:
                download_root.remove(fav)

        download_tree.write(local_path)
        return True
    except Exception as e:
        xbmc.log(f"Merge favourites failed for {folder_name}: {str(e)}", xbmc.LOGERROR)
        show_notification(30003, 5000)  # Fehler beim Mergen der Favoriten
        return False

def sync_standard_favourites():
    if not CUSTOM_FOLDER:
        show_notification(30009, 5000)  # Ein benutzerdefinierter Ordnername ist erforderlich
        return
    if not ftp_folder_exists(f"/{FTP_BASE_PATH}/auto_fav_sync/{CUSTOM_FOLDER}"):
        show_notification(30004, 5000)  # Keine Remote-Favoriten gefunden, Hochladen...
        ftp_upload(LOCAL_FAVOURITES, FTP_PATH)
    else:
        show_notification(30001, 5000)  # Favoriten werden heruntergeladen...
        if ftp_download(FTP_PATH, TEMP_DOWNLOAD_PATH):
            if merge_favourites(LOCAL_FAVOURITES, TEMP_DOWNLOAD_PATH, CUSTOM_FOLDER):
                show_notification(30002, 5000)  # Favoriten heruntergeladen und gemerged
                ftp_upload(LOCAL_FAVOURITES, FTP_PATH)
            else:
                show_notification(30003, 5000)  # Fehler beim Mergen der Favoriten

def sync_static_favourites():
    for folder in STATIC_FOLDERS:
        local_path = os.path.join(SUPER_FAVOURITES_PATH, folder, 'favourites.xml')
        remote_path = f"/{FTP_BASE_PATH}/auto_fav_sync/{CUSTOM_FOLDER}/{folder}/favourites.xml"
        static_server_path = f"/{FTP_BASE_PATH}/auto_fav_sync/static/{folder}/favourites.xml"
        temp_path = os.path.join(xbmcvfs.translatePath('special://temp'), f'favourites_{folder}_download.xml')

        # Erstellen der statischen Ordner im CUSTOM_FOLDER, wenn nicht vorhanden
        if not ftp_folder_exists(f"/{FTP_BASE_PATH}/auto_fav_sync/{CUSTOM_FOLDER}/{folder}"):
            create_ftp_folder(f"/{FTP_BASE_PATH}/auto_fav_sync/{CUSTOM_FOLDER}/{folder}")

        if SERVER_PRIORITY:
            show_notification(30010, 5000, folder=folder)  # Downloading {folder} favourites from server priority folder...
            if ftp_download(static_server_path, temp_path):
                # Wenn server_priority aktiviert ist, überschreiben wir die lokale Datei direkt
                try:
                    xbmcvfs.copy(temp_path, local_path)
                    show_notification(30011, 5000, folder=folder)  # Overwritten {folder} favourites from server
                    ftp_upload(local_path, remote_path)
                except Exception as e:
                    show_notification(30012, 5000, folder=folder)  # Error overwriting {folder} favourites from server
            else:
                show_notification(30013, 5000, folder=folder)  # Error downloading {folder} favourites from server
        else:
            show_notification(30001, 5000)  # Favoriten werden heruntergeladen...
            if ftp_download(remote_path, temp_path):
                if merge_favourites(local_path, temp_path, folder):
                    show_notification(30002, 5000)  # Favoriten heruntergeladen und gemerged
                    ftp_upload(local_path, remote_path)
                else:
                    show_notification(30003, 5000)  # Fehler beim Mergen der Favoriten
            else:
                show_notification(30004, 5000)  # Keine Remote-Favoriten gefunden, Hochladen...
                ftp_upload(local_path, remote_path)

if ENABLED:
    if SYNC_STANDARD:
        sync_standard_favourites()
    if SYNC_STATIC:
        sync_static_favourites()